#include "bspline.h"
#include "model/transformer.h"
#include <QDebug>

Bspline::Bspline() : Shape(BSPLINE)
{
}

Bspline::Bspline(QString id, const QList<Point> &points) : Shape(BSPLINE) {
        this->points = points;
        this->name = "sp_" + id;
}

    //metodo que adiciona pontos ao poligono
void Bspline::addRPoint(const Point &p) {
       this->points << p;
}

//metodo que calcula as coordenadas de ponto de projeção do objeto
void Bspline::calcCpp(double angle)
{
    Point origem;
    origem.setX(0);
    origem.setY(0);
    QList<Point> n = Transformer::rotation(points, origem, angle);
    QList<Point> l;
    for(int i = 0; i < n.size(); i++) {
        Point p = points.at(i);
        Point pn = n.at(i);
        p.setXcpp(pn.getX());
        p.setYcpp(pn.getY());
        l.append(p);
    }
    setRPoints(l);
}

//metodo que retorna a lista de pontos que formam a curva
QList<Point> Bspline::generatePoints(double acurracy) {
    QList<Point> ress;
     double mbs[4][4] = {{-1.0/6,  1.0/2,   -1.0/2, 1.0/6},
                         { 1.0/2,    -1,     1.0/2, 0},
                         {-1.0/2,     0,     1.0/2, 0},
                         { 1.0/6,    4.0/6,  1.0/6, 0}};

     double f0[2], deltaF0[2], delta2F0[2], delta3F0[2];
     double t2 = acurracy * acurracy;
     double t3 = acurracy*t2;

     double matrixCoef[4][2];
     for(int i =0; i<points.size()-3; i++)
     {
         for (int z = 0; z < 4; z++) {
             matrixCoef[z][0] = points.value(z+i).getXcpp();
             matrixCoef[z][1] = points.value(z+i).getYcpp();
         }
         double coef[4][2];

         Transformer::MatMultMat42(mbs,matrixCoef, coef);

         for(int j = 0; j < 2; j++)
         {
             f0[j] = coef[3][j];
             deltaF0[j]  = (coef[0][j]*t3) + (coef[1][j]*t2) + (coef[2][j]*acurracy);
             delta2F0[j] = (6*coef[0][j]*t3) + (2*coef[1][j]*t2);
             delta3F0[j] = (6*coef[0][j]*t3);
         }

         for(double p = 0; p<=1; p+= acurracy)
         {
             Point p;
             p.setXcpp(f0[0]);
             p.setYcpp(f0[1]);
             ress.append(p);
             f0[0] += deltaF0[0]; deltaF0[0] += delta2F0[0]; delta2F0[0] += delta3F0[0];
             f0[1] += deltaF0[1]; deltaF0[1] += delta2F0[1]; delta2F0[1] += delta3F0[1];
         }
     }
    return ress;
}

//metodo que atribui a lista de pontos da curva
void Bspline::setRPoints(QList<Point> points){
    this->points = points;
}

QList<Point> Bspline::getRPoints() {
    return points;
}

void Bspline::saveXML(ofstream* f)
{
    std::stringstream ss;
    char saida[7];
    char p[2];
    p[0] = 0;
    p[1] = 0;
    p[2] = 0;
    if(this->R != 0){
        ss << std::hex << this->R;
        ss >> p;
        saida[0] = p[0];
        saida[1] = p[1];
        ss.clear();
    }else{
     saida[0] = '0';
     saida[1] = '0';
    }
    if(this->G != 0){
        ss << std::hex << this->G;
        ss >> p;
        saida[2] = p[0];
        saida[3] = p[1];
        ss.clear();
    }else
    {
        saida[2] = '0';
        saida[3] = '0';
    }
    if(this->B != 0){
        ss << std::hex<< this->B;
        ss >> p;
        saida[4] = p[0];
        saida[5] = p[1];
    }else
    {
        saida[4] = '0';
        saida[5] = '0';
    }
    saida[6] = 0;
    (*f)<<"\n\t<curvaBSpline nome=\"";
    (*f)<<this->name.toStdString();
    (*f)<<"\" cor=\"";
    (*f)<<saida<<"\">\n";
    QList <Point> pontos = this->getRPoints();
    for(int i =0; i<pontos.length(); i++)
    {
         Point p = pontos.at(i);
         p.xmlPonto(f);
    }
    (*f)<<"\t</curvaBSpline>";
}
