#include "Bezier.h"
#include "transformer.h"
#include <QDebug>
//metodo construtor
Bezier::Bezier() : Shape(BEZIER)
{

}

Bezier::Bezier(QString id, const QList<Point> &points) : Shape(BEZIER) {
        this->points = points;
        this->name = "bez_" + id;
}

//metodo destrutor
Bezier::~Bezier() {
}

//metodo que adiciona pontos ao poligono
void Bezier::addRPoint(const Point &p) {
       this->points << p;
}

//metodo que calcula as coordenadas de ponto de projeção do objeto
void Bezier::calcCpp(double angle)
{
    Point origem;
    origem.setX(0);
    origem.setY(0);
    QList<Point> n = Transformer::rotation(points, origem, angle);
    QList<Point> l;
    for(int i = 0; i < n.size(); i++) {
        Point p = points.at(i);
        Point pn = n.at(i);
        p.setXcpp(pn.getX());
        p.setYcpp(pn.getY());
        l.append(p);
    }
    setRPoints(l);
}

//metodo que retorna a lista de pontos que formam a curva
QList<Point> Bezier::generatePoints(double acurracy) {
    QList<Point> ress;
    Point p1 = points.value(0);
    Point p2 = points.value(1);
    Point p3 = points.value(2);
    Point p4 = points.value(3);


    for (float t = 0; t <= 1; t += acurracy) {
        double x,y;

        x = ((((1-t)*(1-t))*(1-t))*p1.getXcpp()) + (3*t*((1-t)*(1-t))*p2.getXcpp()) + (3*t*t*(1-t)*p3.getXcpp()) + (t*t*t*p4.getXcpp());
        y = ((((1-t)*(1-t))*(1-t))*p1.getYcpp()) + (3*t*((1-t)*(1-t))*p2.getYcpp()) + (3*t*t*(1-t)*p3.getYcpp()) + (t*t*t*p4.getYcpp());
        Point p;
        p.setXcpp(x);
        p.setYcpp(y);
        ress.append(p);
    }
    return ress;
}

//metodo que atribui a lista de pontos da curva
void Bezier::setRPoints(QList<Point> points){
    this->points = points;
}

QList<Point> Bezier::getRPoints() {
    return points;
}

void Bezier::saveXML(ofstream* f)
{
    std::stringstream ss;
    char saida[7];
    char p[2];
    p[0] = 0;
    p[1] = 0;
    p[2] = 0;
    if(this->R != 0){
        ss << std::hex << this->R;
        ss >> p;
        saida[0] = p[0];
        saida[1] = p[1];
        ss.clear();
    }else{
     saida[0] = '0';
     saida[1] = '0';
    }
    if(this->G != 0){
        ss << std::hex << this->G;
        ss >> p;
        saida[2] = p[0];
        saida[3] = p[1];
        ss.clear();
    }else
    {
        saida[2] = '0';
        saida[3] = '0';
    }
    if(this->B != 0){
        ss << std::hex<< this->B;
        ss >> p;
        saida[4] = p[0];
        saida[5] = p[1];
    }else
    {
        saida[4] = '0';
        saida[5] = '0';
    }
    saida[6] = 0;
    (*f)<<"\n\t<curvaBezier nome=\"";
    (*f)<<this->name.toStdString();
    (*f)<<"\" cor=\"";
    (*f)<<saida <<"\">";

    QList <Point> pontos = this->getRPoints();
    int geometrias =floor(pontos.length()/4);
    for(int i =0; i<geometrias; i++)
    {
        (*f)<<"\n\t\t<geometria>\n";
        for(int j= 1; j<=4; j++){
            Point p = pontos.at(i*4+j-1);
            (*f)<<"\t\t\t<p"<<j<< " x=\"";
            (*f)<< p.getX();
            (*f)<<"\" y=\"";
            (*f)<< p.getY();
            (*f)<<"\" z=\"0\"/>\n";
        }
        (*f)<<"\t\t</geometria>\n";
    }
    (*f)<<"\t</curvaBezier>";
}
