#ifndef MEDIATOR_H_
#define MEDIATOR_H_

#include <QList>
#include <QString>
#include <QObject>
#include <QString>
#include <QDebug>
#include <iostream>
#include <fstream>
using namespace std;
#include "model/Shape.h"
#include "model/DisplayFile.h"
#include "model/ShapeFactory.h"
#include "xmlloader.h"



class JanelaPrincipal;
#include "gui/janelaprincipal.h"

class Window;
#include "model/Window.h"

/**
 * \brief Classe mediador, também um singleton. A instância dessa classe é
 * responsável por receber e tratar as requisições vindas dos objetos de
 * interface gráfica.Conhecido como fachada do sistema.
 */
class Mediator {
public:
        // Obtém a instância (única) dessa classe
	static Mediator* instance();

	virtual ~Mediator();

        //metodo que retorna a lista dos objetos presentes no displayFile
	QList<Shape*> getShapes();
        //metodo que chama o metodo para a criação do ponto
        void addPoint(double,double,QString);
        void addPoint(double,double,QString,int ,int , int);
        //metodo que chama o metodo para a criação da linha
        void addLine(double,double,double,double,QString);
        void addLine(double,double,double,double,QString,int ,int , int);
        //metodo que chama o metodo para a criação do poligono
        void addPolygon(QList<double>, QList<double>, QString,bool);
        void addPolygon(QList<double>, QList<double>, QString,bool,int ,int , int);
        //metodo que chama o metodo para a criação da bezier
        void addBezier(QList<double>, QList<double>, QString);
        void addBezier(QList<double>, QList<double>, QString,int ,int , int);
        //metodo que chama o metodo para a criação da spline
        void addBspline(QList<double>, QList<double>, QString);
        void addBspline(QList<double>, QList<double>, QString,int ,int , int);
        //metodo que chama o metodo para realizar o zoomIn da window
        void zoomIn(double);
        //metodo que chama o metodo para realizar o zoomOut da window
        void zoomOut(double);
        //metodo que chama o metodo para realizar a movimentação horizontal(cima baixo conforme o valor)
        void verticalMove(int);
        //metodo que chama o metodo para realizar a movimentação vertical(direita esquerda conforme o valor)
        void horizontalMove(int);
        //metodo que atribui o item selecionado do display file a uma variavel
        void setSelectedItem(Shape*);
        //metodo que retorna o item selecionado do display file
        Shape* getSelectedItem();
        //variavel que armazena o display file
        DisplayFile *displayFile;
        //metodo que chama o metodo que realiza o redesenho dos objetos
        void redraw();
         //metodo que realiza a rotação da window
        void rotate(double);
         //metodo que retorna a instancia da window
        Window* getWindow();

        void loadFile(QString);
        void saveFile(QString);

private:
	Mediator();
	Mediator(const Mediator&);
	Mediator& operator=(const Mediator&);
        //variavel que armazena o item selecionado no display file
        Shape *selectedItem;
        //variavel que armazena a window
        Window *window;
        //variavel que armazena a janela principal
	JanelaPrincipal *janelaPrincipal;
        void saveShapes(FILE*);
};

#endif /* MEDIATOR_H_ */
