#include "Point.h"
#include "transformer.h"


//metodo construtor
Point::Point() : Shape(POINT) {
}

Point::Point(double x, double y) : Shape(POINT) {
	this->x = x;
	this->y = y;
}

Point::Point(QString id, double x, double y) : Shape(POINT) {
	Point(x,y);
	this->name = "p_" + id;
}
//metodo destrutor
Point::~Point() {

}
//metodo que retorna a coordenada x do ponto
double Point::getX() const {
	return x;
}

///metodo que seta o X
void Point::setX(double x) {
    this->x = x;
}



//metodo que retorna a coordenada y do ponto
double Point::getY() const {
	return y;
}

///metodo que seta o Y
void Point::setY(double y) {
    this->y = y;
}

///metodo que retorna a coordenada x dos pontos de projeção
double Point::getXcpp()
{
    return xcpp;
}

///metodo que retorna a coordenada y dos pontos de projeção
double Point::getYcpp()
{
    return ycpp;
}

///metodo que atribui a coordenada x dos pontos de projeção
void Point::setXcpp(double x)
{
    xcpp = x;
}

///metodo que atribui a coordenada y dos pontos de projeção
void Point::setYcpp(double y)
{
    ycpp = y;
}

//metodo que calculo a coordenada de ponto de projeção do objeto
void Point::calcCpp(double angle)
{
    QList<Point> l;
    l.append((*this));
    Point origem;
    origem.setX(0);
    origem.setY(0);
    QList<Point> n = Transformer::rotation(l, origem, angle);
    xcpp = n.at(0).getX();
    ycpp = n.at(0).getY();
}

//metodo que compara se um ponto é igual a outro ponto em questão
bool Point::operator==(const Point &other) const {
	return (this->x == other.x && this->y == other.y);
}

//metodo que compara se um ponto é diferente a outro ponto em questão
bool Point::operator!=(const Point &other) const {
	return !(*this == other);
}
//metodo que soma dois pontos
Point Point::operator+(const Point &other) const {
	Point sum ((this->x + other.x), (this->y + other.y));
	return sum;
}

//metodo que diminui dois pontos
Point Point::operator-(const Point &other) const {
	Point diff ((this->x - other.x), (this->y - other.y));
	return diff;
}

void Point::saveXML(ofstream* f)
{
    std::stringstream ss;
    char cor[7];
    char p[2];
    p[0] = 0;
    p[1] = 0;
    p[2] = 0;
    if(this->R != 0){
        ss << std::hex << this->R;
        ss >> p;
        cor[0] = p[0];
        cor[1] = p[1];
        ss.clear();
    }else{
     cor[0] = '0';
     cor[1] = '0';
    }
    if(this->G != 0){

        ss << std::hex << this->G;
        ss >> p;
        cor[2] = p[0];
        cor[3] = p[1];
        ss.clear();
    }else
    {
        cor[2] = '0';
        cor[3] = '0';
    }
    if(this->B != 0){
        ss << std::hex<< this->B;
        ss >> p;
        cor[4] = p[0];
        cor[5] = p[1];
    }else
    {
        cor[4] = '0';
        cor[5] = '0';
    }
    cor[6] = 0;
    (*f)<<"\n\t<ponto nome=\"";
    (*f)<<this->name.toStdString();
    (*f)<<"\" cor=\"";
    (*f)<<cor;
    (*f)<<"\" x=\""<<this->x;
    (*f)<<"\" y=\"";
    (*f)<< this->y;
    (*f)<<"\" z=\"0\"/>\n";
}

void Point::xmlPonto(ofstream* f)
{
    (*f)<<"\t\t<ponto x=\"";
    (*f)<< this->x;
    (*f)<<"\" y=\"";
    (*f)<< this->y;
    (*f)<<"\" z=\"0\"/>\n";
}
